/*
 * Created on 22-jun-2005
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.gvsig.datalocator.gui;

import java.awt.event.ItemListener;

import java.util.Comparator;
import java.util.TreeSet;
import java.util.Vector;
import java.util.prefs.Preferences;

import javax.swing.JPanel;

import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.instruction.IncompatibleTypesException;
import com.hardcode.gdbms.engine.values.BooleanValue;
import com.hardcode.gdbms.engine.values.Value;
import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiManager.SingletonView;
import com.iver.andami.ui.mdiManager.View;
import com.iver.andami.ui.mdiManager.ViewInfo;
import com.iver.andami.ui.mdiManager.ViewListener;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.FMap;
import com.iver.cit.gvsig.fmap.layers.CancelationException;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.LayerCollectionEvent;
import com.iver.cit.gvsig.fmap.layers.LayerCollectionListener;
import com.iver.cit.gvsig.fmap.layers.LayerPositionEvent;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.Selectable;
import com.iver.cit.gvsig.fmap.layers.layerOperations.VectorialData;
import com.iver.cit.gvsig.fmap.operations.strategies.SelectedZoomVisitor;
import com.iver.cit.gvsig.fmap.operations.strategies.VisitException;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JButton;

/**
 * @author jmorell
 */
public class DataSelectionPanel extends JPanel implements View, ViewListener, SingletonView {
    
	/**
     * 
     */
    private static final long serialVersionUID = 1L;
    private JComboBox jComboBox = null;
	private JLabel jLabel = null;  //  @jve:decl-index=0:visual-constraint="597,16"
	private JLabel jLabel1 = null;  //  @jve:decl-index=0:visual-constraint="873,44"
	private JComboBox jComboBox1 = null;
	private JLabel jLabel2 = null;  //  @jve:decl-index=0:visual-constraint="847,16"
	private JComboBox jComboBox2 = null;
    private ViewInfo viewInfo = null;
	private JButton jButton = null;
	private JButton jButton1 = null;
	private FLayer layerToZoom = null;
	private int fieldToZoomIndex = 0;
	private Value itemToZoom = null;
	private FMap mapContext = null;
	private Preferences prefUsuario = null;
	private LayersListener layersListener = null;
    private Vector vectorialLayers = null;
	
	/**
	 * This method initializes 
	 * 
	 */
	public DataSelectionPanel(FMap mapContext) {
		super();
		this.mapContext = mapContext;
		initializeVectorialLayers();
        initializeLayerToZoom();
		initializeFieldToZoomIndex();
        initialize();
	}
    private void initializeVectorialLayers() {
        vectorialLayers = new Vector();
        for (int i=0;i<mapContext.getLayers().getLayersCount();i++) {
            if (mapContext.getLayers().getLayer(i) instanceof FLyrVect) vectorialLayers.add(mapContext.getLayers().getLayer(i));
        }
    }
	private void initializeLayerToZoom() {
		prefUsuario = Preferences.userRoot();
		String layerName = prefUsuario.get("LAYERNAME_FOR_DATA_LOCATION", "");
        if (layerName.equals("")) layerToZoom = (FLayer)vectorialLayers.get(0);
        boolean layerFound = false;
        for (int i=0;i<vectorialLayers.size();i++) {
            if (((FLayer)vectorialLayers.get(i)).getName().equals(layerName)) {
                layerFound = true;
                layerToZoom = (FLayer)vectorialLayers.get(i);
                break;
            }
        }
        if (!layerFound) layerToZoom = (FLayer)vectorialLayers.get(0);
	}
	private void initializeFieldToZoomIndex() {
        fieldToZoomIndex = prefUsuario.getInt("FIELDINDEX_FOR_DATA_LOCATION", 0);
		AlphanumericData lyr = (AlphanumericData)layerToZoom;
		DataSource ds;
        try {
            ds = lyr.getRecordset();
            if (fieldToZoomIndex > (ds.getFieldCount()-1)) fieldToZoomIndex = 0;
        } catch (DriverException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
	}
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
        jLabel2 = new JLabel();
        jLabel1 = new JLabel();
        jLabel = new JLabel();
        this.setLayout(null);
        this.setSize(286, 161);
        jLabel.setBounds(6, 6, 43, 23);
        jLabel.setText(PluginServices.getText(this,"Capa") + ":");
        jLabel1.setBounds(6, 34, 43, 23);
        jLabel1.setText(PluginServices.getText(this,"Campo") + ":");
        jLabel2.setBounds(6, 61, 43, 23);
        jLabel2.setText(PluginServices.getText(this,"Valor") + ":");
        this.add(getJComboBox(), null);
        this.add(jLabel, null);
        this.add(jLabel1, null);
        this.add(getJComboBox1(), null);
        this.add(jLabel2, null);
        this.add(getJComboBox2(), null);
        this.add(getJButton(), null);
        this.add(getJButton1(), null);
        layersListener = new LayersListener(mapContext.getLayers());
		mapContext.getLayers().addLayerCollectionListener(layersListener);
	}
	private String[] getLayerNames() {
		String[] layerNames = new String[vectorialLayers.size()];
	    for (int i=0;i<vectorialLayers.size();i++) {
	        layerNames[i] = ((FLayer)vectorialLayers.get(i)).getName();
	    }
	    return layerNames;
	}
	private String[] getFieldNames() {
		AlphanumericData lyr = (AlphanumericData)layerToZoom;
		DataSource ds;
        String[] fieldNames = null;
		try {
            ds = lyr.getRecordset();
			fieldNames = new String[ds.getFieldCount()];
			for (int i = 0; i < ds.getFieldCount(); i++) {
				fieldNames[i] = ds.getFieldName(i);
			}
        } catch (DriverException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
		return fieldNames;
	}
	private Value[] getNewValues() {
		AlphanumericData lyr = (AlphanumericData)layerToZoom;
		DataSource ds;
		Value[] newValues = null;
		try {
            ds = lyr.getRecordset();
		    Value[] values = new Value[(int)ds.getRowCount()];
			for (int i=0;i<ds.getRowCount();i++) {
			    Value value = ds.getFieldValue(i, fieldToZoomIndex);
			    values[i] = value;
			}
			//Quitar los nombres repetidos y ordenarlos
			TreeSet treeSet = new TreeSet(new Comparator() {
                public int compare(Object o1, Object o2) {
                    Value v1 = (Value) o1;
                    Value v2 = (Value) o2;
                    try {
                        if (((BooleanValue)v1.less(v2)).getValue()){
                            return -1;
                        }else if (((BooleanValue)v1.greater(v2)).getValue()){
                            return 1;
                        }else{
                            return 0;
                        }
                    } catch (IncompatibleTypesException e) {
                        throw new RuntimeException(e);
                    }
                }
            });
			for (int i=0;i<values.length;i++) {
			    treeSet.add(values[i]);
			}
			newValues = (Value[])treeSet.toArray(new Value[0]);
        } catch (DriverException ee) {
            // TODO Auto-generated catch block
            ee.printStackTrace();
        } catch (com.hardcode.gdbms.engine.data.driver.DriverException eee) {
            // TODO Auto-generated catch block
            eee.printStackTrace();
        }
        return newValues;
	}
	private class LayersListener implements LayerCollectionListener {
	    public LayersListener(FLayers layers){
	    }
        /* (non-Javadoc)
         * @see com.iver.cit.gvsig.fmap.layers.LayerCollectionListener#layerAdded(com.iver.cit.gvsig.fmap.layers.LayerCollectionEvent)
         */
        public void layerAdded(LayerCollectionEvent e) {
            initializeVectorialLayers();
            ((ChangeLayerToZoomItemListener)jComboBox.getItemListeners()[0]).setLayers(vectorialLayers);
            jComboBox.removeAllItems();
		    DefaultComboBoxModel defaultModel = new DefaultComboBoxModel(getLayerNames());
		    jComboBox.setModel(defaultModel);
			jComboBox.setSelectedItem(layerToZoom.getName());
        }
        /* (non-Javadoc)
         * @see com.iver.cit.gvsig.fmap.layers.LayerCollectionListener#layerRemoved(com.iver.cit.gvsig.fmap.layers.LayerCollectionEvent)
         */
        public void layerRemoved(LayerCollectionEvent e) {
            initializeVectorialLayers();
            ((ChangeLayerToZoomItemListener)jComboBox.getItemListeners()[0]).setLayers(vectorialLayers);
		    if (vectorialLayers.size()>0) {
	            jComboBox.removeAllItems();
				String[] layerNames = new String[vectorialLayers.size()];
			    boolean currentLayerRemoved = true;
				for (int i=0;i<vectorialLayers.size();i++) {
			        layerNames[i] = ((FLayer)vectorialLayers.get(i)).getName();
			        if (layerToZoom.getName().equals(layerNames[i])) currentLayerRemoved = false;
			    }
			    DefaultComboBoxModel defaultModel = new DefaultComboBoxModel(layerNames);
			    jComboBox.setModel(defaultModel);
			    if (currentLayerRemoved) {
			        layerToZoom = ((FLayer)vectorialLayers.get(0));
	    			defaultModel = new DefaultComboBoxModel(getFieldNames());
	    		    jComboBox1.setModel(defaultModel);
	    		    fieldToZoomIndex = 0;
	    		    jComboBox1.setSelectedIndex(fieldToZoomIndex);
	    			defaultModel = new DefaultComboBoxModel(getNewValues());
	    			jComboBox2.setModel(defaultModel);
	    			itemToZoom = getNewValues()[0];
	    			jComboBox2.setSelectedItem(itemToZoom);
			    }
			    jComboBox.setSelectedItem(layerToZoom.getName());
		    }else {
            	if (PluginServices.getMainFrame() == null)
            		((JDialog) (getParent().getParent().getParent().getParent())).dispose();
            	else
            		PluginServices.getMDIManager().closeView(DataSelectionPanel.this);
		    }
        }
        /* (non-Javadoc)
         * @see com.iver.cit.gvsig.fmap.layers.LayerCollectionListener#layerMoved(com.iver.cit.gvsig.fmap.layers.LayerPositionEvent)
         */
        public void layerMoved(LayerPositionEvent e) {
            // TODO Auto-generated method stub
            
        }
        /* (non-Javadoc)
         * @see com.iver.cit.gvsig.fmap.layers.LayerCollectionListener#layerAdding(com.iver.cit.gvsig.fmap.layers.LayerCollectionEvent)
         */
        public void layerAdding(LayerCollectionEvent e) throws CancelationException {
            // TODO Auto-generated method stub
            
        }
        /* (non-Javadoc)
         * @see com.iver.cit.gvsig.fmap.layers.LayerCollectionListener#layerMoving(com.iver.cit.gvsig.fmap.layers.LayerPositionEvent)
         */
        public void layerMoving(LayerPositionEvent e) throws CancelationException {
            // TODO Auto-generated method stub
            
        }
        /* (non-Javadoc)
         * @see com.iver.cit.gvsig.fmap.layers.LayerCollectionListener#layerRemoving(com.iver.cit.gvsig.fmap.layers.LayerCollectionEvent)
         */
        public void layerRemoving(LayerCollectionEvent e) throws CancelationException {
            // TODO Auto-generated method stub
            
        }
        /* (non-Javadoc)
         * @see com.iver.cit.gvsig.fmap.layers.LayerCollectionListener#activationChanged(com.iver.cit.gvsig.fmap.layers.LayerCollectionEvent)
         */
        public void activationChanged(LayerCollectionEvent e) throws CancelationException {
            // TODO Auto-generated method stub
            
        }
        /* (non-Javadoc)
         * @see com.iver.cit.gvsig.fmap.layers.LayerCollectionListener#visibilityChanged(com.iver.cit.gvsig.fmap.layers.LayerCollectionEvent)
         */
        public void visibilityChanged(LayerCollectionEvent e) throws CancelationException {
            // TODO Auto-generated method stub
            
        }
	}
    /* (non-Javadoc)
     * @see com.iver.andami.ui.mdiManager.View#getViewInfo()
     */
    public ViewInfo getViewInfo() {
        if (viewInfo == null) {
            viewInfo=new ViewInfo(ViewInfo.MODELESSDIALOG);
            viewInfo.setTitle(PluginServices.getText(this,"Localizador_por_atributo"));
        }
        return viewInfo;
    }
	/**
	 * This method initializes jComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJComboBox() {
		if (jComboBox == null) {
			jComboBox = new JComboBox();
            DefaultComboBoxModel defaultModel = new DefaultComboBoxModel(getLayerNames());
            jComboBox.setModel(defaultModel);
			jComboBox.setBounds(54, 6, 218, 23);
			jComboBox.setSelectedItem(layerToZoom.getName());
			ChangeLayerToZoomItemListener changeLayerToZoomItemListener = new ChangeLayerToZoomItemListener(vectorialLayers);
			jComboBox.addItemListener(changeLayerToZoomItemListener);
		}
		return jComboBox;
	}
	private class ChangeLayerToZoomItemListener implements ItemListener {
	    private Vector layers;
	    public ChangeLayerToZoomItemListener(Vector layers) {
	        this.layers = layers;
	    }
		public void itemStateChanged(java.awt.event.ItemEvent e) {
		    if (jComboBox.getItemCount()>0) {
                for (int i=0;i<layers.size();i++) {
                    if (((FLayer)layers.get(i)).getName().equals((String)jComboBox.getSelectedItem())) {
                        layerToZoom = (FLayer)layers.get(i);
                        break;
                    }
                }
    			fieldToZoomIndex = 0;
				prefUsuario.put("LAYERNAME_FOR_DATA_LOCATION", (String)jComboBox.getSelectedItem());
    		    DefaultComboBoxModel defaultModel = new DefaultComboBoxModel(getFieldNames());
    		    jComboBox1.setModel(defaultModel);
    		    defaultModel = new DefaultComboBoxModel(getNewValues());
    			jComboBox2.setModel(defaultModel);
    			jComboBox2.setSelectedIndex(0);
    			itemToZoom = getNewValues()[0];
		    }
		}
        /**
         * @param layers The layers to set.
         */
        public void setLayers(Vector layers) {
            this.layers = layers;
        }
	}
	/**
	 * This method initializes jComboBox1	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJComboBox1() {
		if (jComboBox1 == null) {
			jComboBox1 = new JComboBox();
            DefaultComboBoxModel defaultModel = new DefaultComboBoxModel(getFieldNames());
            jComboBox1.setModel(defaultModel);
			jComboBox1.setBounds(54, 34, 218, 23);
			jComboBox1.setSelectedIndex(fieldToZoomIndex);
			ChangeFieldItemListener changeFieldItemListener = new ChangeFieldItemListener(vectorialLayers);
			jComboBox1.addItemListener(changeFieldItemListener);
		}
		return jComboBox1;
	}
	private class ChangeFieldItemListener implements ItemListener {
	    public ChangeFieldItemListener(Vector layers) {
	    }
		public void itemStateChanged(java.awt.event.ItemEvent e) {    
			String fieldToZoom = ((String)jComboBox1.getSelectedItem());
			AlphanumericData lyr = (AlphanumericData)layerToZoom;
			DataSource ds;
			try {
                ds = lyr.getRecordset();
    			for (int i=0;i<ds.getFieldCount();i++) {
    			    String fieldNamei = ds.getFieldName(i);
    			    if (fieldToZoom.equals(fieldNamei)) {
    			        fieldToZoomIndex = i;
    					prefUsuario.putInt("FIELDINDEX_FOR_DATA_LOCATION", i);
    			    }
    			}
    			Value[] values = new Value[(int)ds.getRowCount()];
    			for (int i=0;i<ds.getRowCount();i++) {
    			    Value value = ds.getFieldValue(i, fieldToZoomIndex);
    			    values[i] = value;
    			}
    			//Quitar los nombres repetidos y ordenarlos
    			TreeSet treeSet = new TreeSet(new Comparator() {
                    public int compare(Object o1, Object o2) {
                        Value v1 = (Value) o1;
                        Value v2 = (Value) o2;
                        try {
                            if (((BooleanValue)v1.less(v2)).getValue()){
                                return -1;
                            }else if (((BooleanValue)v1.greater(v2)).getValue()){
                                return 1;
                            }else{
                                return 0;
                            }
                        } catch (IncompatibleTypesException e) {
                            throw new RuntimeException(e);
                        }
                    }
                });
    			for (int i=0;i<values.length;i++) {
    			    treeSet.add(values[i]);
    			}
    			Value[] newValues = (Value[])treeSet.toArray(new Value[0]);
    		    DefaultComboBoxModel defaultModel = new DefaultComboBoxModel(newValues);
    		    jComboBox2.setModel(defaultModel);
    		    jComboBox2.setSelectedIndex(0);
    		    itemToZoom = newValues[0];
            } catch (DriverException ee) {
                // TODO Auto-generated catch block
                ee.printStackTrace();
            } catch (com.hardcode.gdbms.engine.data.driver.DriverException eee) {
                // TODO Auto-generated catch block
                eee.printStackTrace();
            }
		}
	}
	/**
	 * This method initializes jComboBox2	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJComboBox2() {
		if (jComboBox2 == null) {
			jComboBox2 = new JComboBox();
            DefaultComboBoxModel defaultModel = new DefaultComboBoxModel(getNewValues());
            jComboBox2.setModel(defaultModel);
			jComboBox2.setSelectedIndex(-1);
			jComboBox2.setBounds(54, 61, 218, 23);
			ChangeItemToZoomItemListener changeItemToZoomItemListener = new ChangeItemToZoomItemListener(vectorialLayers);
			jComboBox2.addItemListener(changeItemToZoomItemListener);
		}
		return jComboBox2;
	}
	private class ChangeItemToZoomItemListener implements ItemListener {
	    private Vector layers;
	    public ChangeItemToZoomItemListener(Vector layers) {
	        this.layers = layers;
	    }
		public void itemStateChanged(java.awt.event.ItemEvent e) {    
            for (int i=0;i<layers.size();i++) {
                if (((FLayer)layers.get(i)).getName().equals((String)jComboBox.getSelectedItem())) {
                    layerToZoom = (FLayer)layers.get(i);
                    break;
                }
            }
			itemToZoom = ((Value)jComboBox2.getSelectedItem());
		}
	}
	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJButton() {
		if (jButton == null) {
			jButton = new JButton();
			jButton.setBounds(6, 99, 128, 23);
			jButton.setText(PluginServices.getText(this,"Zoom"));
			jButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
				    if (jComboBox2.getSelectedIndex()!=-1) {
						AlphanumericData lyr = (AlphanumericData)layerToZoom;
						DataSource ds;
						Selectable selectable = (Selectable)layerToZoom;
						FBitSet fBitSet = selectable.getSelection();
						Vector indices = new Vector();
						try {
			                ds = lyr.getRecordset();
			    			for (int i=0;i<ds.getRowCount();i++) {
			    			    Value value = ds.getFieldValue(i, fieldToZoomIndex);
			    			    if (((BooleanValue)itemToZoom.equals(value)).getValue()) {
			    			        indices.add(new Integer(i));
			    			        fBitSet.set(i);
			    			    }
			    			}
			            } catch (DriverException ee) {
			                // TODO Auto-generated catch block
			                ee.printStackTrace();
			            } catch (com.hardcode.gdbms.engine.data.driver.DriverException eee) {
			                // TODO Auto-generated catch block
			                eee.printStackTrace();
			            } catch (IncompatibleTypesException eeee) {
	                        // TODO Auto-generated catch block
	                        eeee.printStackTrace();
	                    }
						selectable.setSelection(fBitSet);
						SelectedZoomVisitor visitor = new SelectedZoomVisitor();
						try {
	                        ((VectorialData)layerToZoom).process(visitor);
	                    } catch (DriverException e1) {
	                        // TODO Auto-generated catch block
	                        e1.printStackTrace();
	                    } catch (VisitException e1) {
	                        // TODO Auto-generated catch block
	                        e1.printStackTrace();
	                    }
			            mapContext.getViewPort().setExtent(visitor.getSelectBound());
			            selectable.clearSelection();
				    } else {
				        System.out.println("Localizador por atributo: El campo valor debe estar inicializado antes de hacer Zoom.");
				    }
				}
			});
		}
		return jButton;
	}
	/**
	 * This method initializes jButton1	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJButton1() {
		if (jButton1 == null) {
			jButton1 = new JButton();
			jButton1.setBounds(139, 99, 128, 23);
			jButton1.setText(PluginServices.getText(this,"Salir"));
			jButton1.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
	            	if (PluginServices.getMainFrame() == null)
	            		((JDialog) (getParent().getParent().getParent().getParent())).dispose();
	            	else
	            		PluginServices.getMDIManager().closeView(DataSelectionPanel.this);
				}
			});
		}
		return jButton1;
	}
    /* (non-Javadoc)
     * @see com.iver.andami.ui.mdiManager.ViewListener#viewActivated()
     */
    public void viewActivated() {
        // TODO Auto-generated method stub
        
    }
    /* (non-Javadoc)
     * @see com.iver.andami.ui.mdiManager.ViewListener#viewClosed()
     */
    public void viewClosed() {
		mapContext.getLayers().removeLayerCollectionListener(layersListener);
    }
    /* (non-Javadoc)
     * @see com.iver.andami.ui.mdiManager.SingletonView#getViewModel()
     */
    public Object getViewModel() {
        // Debe devolver una cadena. Mirar Console del CorePlugin
        return "DataSelectionPanel";
    }
}  //  @jve:decl-index=0:visual-constraint="10,10"
